-- Khmer for VP
-- Author: pineappledan
-- DateCreated: 28/04/2019
--------------------------------------------------------------
print("Khmer VPCompatibility Loaded")

include("FLuaVector.lua")

function PositionCalculator(i1, i2)
	return HexToWorld(ToHexFromGrid(Vector2(i1, i2)))
end

local eCivilizationKhmer = GameInfoTypes.CIVILIZATION_KHMER
local eBarayEffectDummy = GameInfoTypes.BUILDING_KHMER_BARAY_EFFECT
local fGameSpeedModifier = GameInfo.GameSpeeds[ Game.GetGameSpeedType() ].ConstructPercent / 100

function GreatBaraysOfAngkorWonder(iPlayer, iCity, iBuilding)
	local pPlayer = Players[iPlayer]
	local pCity = pPlayer:GetCityByID(iCity)
	local building = GameInfo.Buildings[iBuilding]

	if (GameInfo.BuildingClasses[building.BuildingClass].MaxGlobalInstances == (1)) then
	-- All civs, including non-Khmer civs, boost Phnom Preah by 2 culture if they have a wonder in the city
		pCity:SetNumRealBuilding(GameInfoTypes.BUILDING_KHMER_WONDER_DUMMY, 1)
		print("This City has a Wonder in it!")
	
		if (pPlayer and pPlayer:GetCivilizationType() == eCivilizationKhmer) then
	
	-- Attempt 3: Scales with City food production
			local iChange1 = math.max(40, math.floor(3 * pCity:GetYieldRate( YieldTypes.YIELD_FOOD )))
			pCity:SetNumRealBuilding(eBarayEffectDummy, (pCity:GetNumRealBuilding(eBarayEffectDummy)+1))
			pCity:ChangeFood(iChange1)
			pPlayer:ChangeFaith(iChange1)

			if pPlayer:IsHuman() and pPlayer:IsTurnActive() then
				local vCityPosition = PositionCalculator(pCity:GetX(), pCity:GetY())
				local sName = pCity:GetName()

				Events.AddPopupTextEvent(vCityPosition, "[COLOR_YIELD_FOOD]+"..iChange1.." [ICON_FOOD][ENDCOLOR]", 1)
				Events.AddPopupTextEvent(vCityPosition, "[COLOR_WHITE]+"..iChange1.." [ICON_PEACE][ENDCOLOR]", 1.5)
				pPlayer:AddNotification(NotificationTypes.NOTIFICATION_INSTANT_YIELD,
					'Wonder completed:[NEWLINE][ICON_BULLET][COLOR_POSITIVE_TEXT]'..sName..': [ENDCOLOR]+'..iChange1..' [ICON_FOOD] Food[NEWLINE][ICON_BULLET][COLOR_POSITIVE_TEXT]'..sName..': [ENDCOLOR]+'..iChange1..' [ICON_PEACE] Faith',
					'Bonus Yields in '..sName,
					pCity:GetX(), pCity:GetY(), pCity:GetID())
			end
		end	
	end
	if (pPlayer and pPlayer:GetCivilizationType() == eCivilizationKhmer) then
		-- +Great Engineers (but no food or faith) when a national wonder is completed
		if (building.NationalPopRequired > 1 or building.IsReformation == 1) then
			pCity:SetNumRealBuilding(eBarayEffectDummy, (pCity:GetNumRealBuilding(eBarayEffectDummy)+1))
			print("National Wonder Constructed")
		end
	end
end

function PrangPopEffect(iPlayer)
	local pPlayer = Players[iPlayer]
		
	if (pPlayer and pPlayer:GetCivilizationType() == eCivilizationKhmer) then
		
		--Give each Khmer city GEngineer Points based on the number of Baray effect dummies in that city 
		--Dummies are added every time a World Wonder, National Wonder, or Public Works project is done
		--Add an additional +1 GEng point for the Palace (counted as a national wonder for the UA)
		--Set the production on Phnom Preah based on the Population of the city. works for all civs.
		for city in pPlayer:Cities() do
			local eEngineer = GameInfo.Specialists.SPECIALIST_ENGINEER.ID
			local CapitalYesNo = 0
			if city:IsCapital() then 
				CapitalYesNo = 1
			end 
			local pGEngChange = ((100 + city:GetGreatPeopleRateModifier() + city:GetSpecialistCityModifier(eEngineer)) 
								* (city:GetNumRealBuilding(eBarayEffectDummy) + CapitalYesNo))
			city:SetNumRealBuilding(GameInfoTypes.BUILDING_KHMER_POP_DUMMY, math.floor(city:GetPopulation()*0.334))
			city:ChangeSpecialistGreatPersonProgressTimes100(eEngineer, pGEngChange)
		end
	else
		for city in pPlayer:Cities() do
			city:SetNumRealBuilding(GameInfoTypes.BUILDING_KHMER_POP_DUMMY, math.floor(city:GetPopulation()*0.334))
		end
	end
end

function GreatBaraysOfAngkorPubWorks(iPlayer, iCity, iProject)
	local pPlayer = Players[iPlayer]
	if not(pPlayer and pPlayer:GetCivilizationType() == eCivilizationKhmer) then return end
	
	if iProject == GameInfo.Projects.PROJECT_PUBLIC_WORKS.ID then
	-- building a public works project triggers the food, faith, and GEngineer points from the UA, identical to a wonder 
		local pCity = pPlayer:GetCityByID(iCity)
		local iEraModifier = math.max(pPlayer:GetCurrentEra(), 1)
		local iChange2 = math.max(40, math.floor(2 * pCity:GetYieldRate( YieldTypes.YIELD_FOOD )))
		pCity:SetNumRealBuilding(eBarayEffectDummy, (pCity:GetNumRealBuilding(eBarayEffectDummy)+1))
		pCity:ChangeFood(iChange2)
		pPlayer:ChangeFaith(iChange2)

		if pPlayer:IsHuman() and pPlayer:IsTurnActive() then
			local vCityPosition = PositionCalculator(pCity:GetX(), pCity:GetY())
			local sName = pCity:GetName()

			Events.AddPopupTextEvent(vCityPosition, "[COLOR_YIELD_FOOD]+"..iChange2.." [ICON_FOOD][ENDCOLOR]", 1)
			Events.AddPopupTextEvent(vCityPosition, "[COLOR_WHITE]+"..iChange2.." [ICON_PEACE][ENDCOLOR]", 1.5)
			pPlayer:AddNotification(NotificationTypes.NOTIFICATION_INSTANT_YIELD,
				'Public Works Project completed:[NEWLINE][ICON_BULLET][COLOR_POSITIVE_TEXT]'..sName..': [ENDCOLOR]+'..iChange2..' [ICON_FOOD] Food[NEWLINE][ICON_BULLET][COLOR_POSITIVE_TEXT]'..sName..': [ENDCOLOR]+'..iChange2..' [ICON_PEACE] Faith',
				'Bonus Yields in '..sName,
				pCity:GetX(), pCity:GetY(), pCity:GetID())
		end
	end
end


if Game.IsCivEverActive(eCivilizationKhmer) then
	GameEvents.CityConstructed.Add(GreatBaraysOfAngkorWonder)
	GameEvents.CityProjectComplete.Add(GreatBaraysOfAngkorPubWorks)
	GameEvents.PlayerDoTurn.Add(PrangPopEffect)
end